﻿# Функции взимодействия с confluence

## Введение

Модуль предоставляет набор функционала для взаимодействия с Atlassian Confluence посредством REST-API.
Описание продукта по ссылке `https://ru.atlassian.com/software/confluence`

## Использование

Все методы модуля требуют параметры подключения к сервереу confluence: Адрес и учетные данные.

Для начала необходимо сформировать описание подключения методом `ОписаниеПодключения`

### Примеры

#### Подключение

`Подключение = Confluence.ОписаниеПодключения("https://confluence.myserver.ru", "user", "password")`.

#### Создание страницы

`ИдентификаторНовойСтраницы = Confluence.СоздатьСтраницу(Подключение, "SpaceKey", "Новая страница", "Содержимое новой страницы")`

#### Переименование страницы

`ИдентификаторНовойСтраницы = Confluence.ОбновитьСтраницу(Подключение, "SpaceKey", "Новое имя страницы", ИдентификаторНовойСтраницы)`

#### Удаление страницы

`Confluence.УдалитьСтраницу(Подключение, "SpaceKey", "Новое имя страницы")`

#### Установка метки странице

`Confluence.ПрикрепитьМеткуКСтранице(ПараметрыПодключения, ИдентификаторСтраницы, "Моя метка")`

### Примеры нового API

```bsl

// Настройки подключения
Подключение = Confluence.ОписаниеПодключения("https://confluence.myserver.ru", "user", "password")

// Получение идентификатора страницы
ИдентификаторРодителя = Confluence.НайтиСтраницуПоИмени(Подключение, "SpaceKey", "Корневая страница");

// Структура адреса страницы (Пространство, Наименование, Идентификатор, Родитель)
АдресСтраницы = Confluence.АдресСтраницы("SpaceKey", "Новая страница", , ИдентификаторРодителя);

// Создадим новую страницу
Confluence.Создать(Подключение, АдресСтраницы, "Содержимое страницы");
Сообщить("Идентификатор новой страницы: " + АдресСтраницы.Идентификатор)

// Обновим содержимое
Confluence.Обновить(Подключение, АдресСтраницы, "Новое содержимое страницы", Истина);

// Адрес второй страницы
АдресСтраницы = Confluence.АдресСтраницы("SpaceKey", "Вторая страница", , ИдентификаторРодителя);

// Обновим если содержимое отличается, либо создадим если отсутствует
Confluence.СоздатьИлиОбновить(Подключение, АдресСтраницы, "Содержимое второй страницы");

// Опубликуем страницу в формате Markdown
АдресСтраницы = Confluence.АдресСтраницы("SpaceKey", "Третья страница Markdown", , ИдентификаторРодителя);
Содержимое = Новый Структура("Значение, Формат", "# Содержимое третьей страницы", confluence.ФорматыСодержимого.MarkDown);
Confluence.СоздатьИлиОбновить(Подключение, АдресСтраницы, Содержимое);


```
